/*  
Theme Name: BuddyPress Default Member Theme
Theme URI: http://buddypress.org
Description: The default member theme for BuddyPress. This theme handles all non-blog pages.
Version: 1.0.2
Author: Andy Peatling
Author URI: http://apeatling.wordpress.com
*/

@import url(css/loader.php);