/*  
Theme Name: BuddyPress Default Home Theme
Theme URI: http://buddypress.com/
Description: A widget framework theme for the home of a BuddyPress enabled installation. This theme is not required to use BuddyPress.
Version: 1.0.2
Author: Andy Peatling
Author URI: http://apeatling.wordpress.com
*/

/* Load the base styles */
@import url(css/base.css);

/***
 * Rename /css/custom-sample.css to /css/custom.css and uncomment the following line to load your own
 * custom styles without modifying the originals (this will preserve your changes across theme updates)
 *  
 * @import url(css/custom.css);
 */

/***
 * NOTE: If you rename /css/site-wide-sample.css to /css/site-wide.css you can apply CSS styles to any
 * theme that is in use across your whole site. This includes any blog or member theme. You do not
 * need to add an @import declaration in this file, it will load automaticaly.
 */