body { padding-top: 28px !important; }

#wp-admin-bar {
	position: absolute !important;
	top: 0 !important;
	left: 0 !important;
	width: 100% !important;
	z-index: 1001 !important;
	height: 28px !important;
	color: #fff !important;
	text-align: left !important;
	background:url(../images/admin_bar_back.gif) #818181 repeat-x !important;
	font: 12px normal "Lucida Grande", "Lucida Sans Unicode", "Lucida Sans", Tahoma, Verdana, Arial, sans-serif !important;
	font-weight: normal !important;
}

#wp-admin-bar * { z-index: 999 !important; }

#wp-admin-bar img#admin-bar-logo {
	position: absolute !important;
	top: 8px;
	left: 10px;
}

#wp-admin-bar a img {
	border: none !important;
}

#wp-admin-bar li {
	list-style: none !important;
	margin: 0 !important;
}

#wp-admin-bar li * {
	font: 12px normal "Lucida Grande", "Lucida Sans Unicode", "Lucida Sans", Tahoma, Verdana, Arial, sans-serif !important;
	font-weight: normal !important;
	background-image: none !important;
}

#wp-admin-bar li a {
	padding: 5px 25px 7px 15px !important;
	font: 12px normal "Lucida Grande", "Lucida Sans Unicode", "Lucida Sans", Tahoma, Verdana, Arial, sans-serif !important;
	font-weight: normal !important;
}
	#wp-admin-bar li.no-arrow a {
		padding-right: 15px !important;
	}

#wp-admin-bar li:hover, #wp-admin-bar li.hover {
	position: static;
}

/*******************/

#wp-admin-bar ul { /* all lists */
	margin: 0 !important;
	list-style: none !important;
	line-height: 1 !important;
	cursor: pointer !important;
	height: auto !important;
	padding: 0 !important;
}

#wp-admin-bar ul {
	margin-left: 80px !important;
}

#wp-admin-bar ul li { /* all list items */
	padding: 0 !important;
	float: left !important;
	background: url( ../images/admin-menu-arrow.gif ) 88% 53% no-repeat;
	position: relative;
}
	#wp-admin-bar ul li.no-arrow {
		background: none;
	}
	
#wp-admin-bar ul li ul a {
	color: #666 !important;
}

#wp-admin-bar ul li.align-right {
	position: absolute;
	right: 0;
}

#wp-admin-bar ul li a {
	display: block !important;
	font-size: 12px !important;
	font-family: "Lucida Grande", "Lucida Sans Unicode", "Lucida Sans", Tahoma, Verdana, Arial, sans-serif !important;
	text-decoration: none !important;
	color: #fff !important;
}

#wp-admin-bar ul.main-nav li:hover, #wp-admin-bar ul.main-nav li.sfhover, #wp-admin-bar ul.main-nav li ul li.sfhover {
	background-color: #666 !important;
}

#wp-admin-bar ul.main-nav li ul li:hover a, #wp-admin-bar ul.main-nav li ul li.sfhover a {
	background-color: #888 !important;
	color: #fff !important;
}

#wp-admin-bar ul.main-nav li ul li:hover ul li.alt a {
	background-color: #fafafa !important;
	color: #666 !important;
}

#wp-admin-bar ul.main-nav li ul li:hover ul li a {
	background-color: #f4f4f4 !important;
	color: #666 !important;
}

#wp-admin-bar ul.main-nav li ul li:hover ul li:hover a {
	background-color: #888 !important;
	color: white !important;
}

#wp-admin-bar ul.main-nav li ul li ul li:hover a, #wp-admin-bar ul.main-nav li ul li ul li.sfhover a {
	color: #fff !important;
}

/* second-level lists */

#wp-admin-bar ul li ul {
	position: absolute !important;
	width: 185px !important;
	left: -999em !important;
	margin-left: 0 !important;
}

#wp-admin-bar ul li ul li {
	float: left !important;
	width: 183px;
	margin: 0 !important;
	border: none !important;
	border-bottom: 1px solid #f4f4f4 !important;
	background-color: #f4f4f4 !important;
	border-left: 1px solid #ccc !important;
	border-right: 1px solid #ccc !important;
	margin-top: -1px !important;
}

#wp-admin-bar ul li ul li.alt {
	background-color: #fafafa !important;
	border-bottom: 1px solid #fafafa !important;
}

#wp-admin-bar ul li ul li:last-child {
	border-bottom: 1px solid #ccc !important;
	-moz-border-radius-bottomleft: 4px !important;
	-khtml-border-bottom-left-radius: 4px !important;
	-webkit-border-bottom-left-radius: 4px !important;
	border-bottom-left-radius: 4px !important;
	-moz-border-radius-bottomright: 4px !important;
	-khtml-border-bottom-right-radius: 4px !important;
	-webkit-border-bottom-right-radius: 4px !important;
	border-bottom-right-radius: 4px !important;
}

#wp-admin-bar ul.main-nav li ul li a:hover, #wp-admin-bar ul.main-nav li.sfhover ul li a:hover {
	background-color: #888 !important;
}

#wp-admin-bar ul li div.admin-bar-clear {
	clear: both !important;
	background-color: #eee !important;
	border-bottom: 3px solid #eee !important;
}

/* third-and-above-level lists */

#wp-admin-bar ul li ul ul {
	margin: -28px 0 0 183px !important;
}

#wp-admin-bar ul li:hover ul, #wp-admin-bar ul li li:hover ul, #wp-admin-bar ul li.sfhover ul, #wp-admin-bar ul li ul li.sfhover ul { /* lists nested under hovered list items */
	left: auto !important;
}

#wp-admin-bar ul li.align-right:hover ul {
	right: 0 !important;
}

#wp-admin-bar ul li:hover ul ul, #wp-admin-bar li.sfhover ul li ul {
	left: -999em !important;
}

#wp-admin-bar ul li ul li ul li:first-child {
	border-top: 1px solid #ccc !important;
	-moz-border-radius-topright: 4px !important;
	-khtml-border-top-right-radius: 4px !important;
	-webkit-border-top-right-radius: 4px !important;
	border-top-right-radius: 4px !important;
	margin-top: -2px !important;
}

/* Menu item css */

#wp-admin-bar img.avatar {
	float: left !important;
	border: 2px solid #fff !important;
	margin-right: 8px !important;
}

#wp-admin-bar span.activity {
	display: block !important;
	color: #888 !important;
	margin-left: 34px !important;
	font-size: 10px !important;
	background: none !important;
	border: none !important;
	padding: 0 !important;
}

#wp-admin-bar li a:hover span.activity {
	color: #ccc !important;
}

#wp-admin-bar ul.author-list li {
	height: 55px !important;
}

#wp-admin-bar ul li#bp-adminbar-notifications-menu a span {
	background: #fff !important;
	padding: 0 6px !important;
	color: #555 !important;
	font-weight: bold !important;
	font-size: 0.8em !important;
	-moz-border-radius: 2px !important;
	-khtml-border-radius: 2px !important;
	-webkit-border-radius: 2px !important;
	margin-left: 2px !important;
} 